#!/usr/bin/perl -w

#   The PerlGP Genetic Programming System
#   Copyright (C) 2003 Bob MacCallum
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License ($PERLGP_BASE/LICENSE) for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   The original author's contact details are:
#   Bob MacCallum
#   Stockholm Bioinformatics Center
#   Stockholm University, 106 91 Stockholm, Sweden.
#   WWW: http://perlgp.org  EMAIL: info AT perlgp DOT org


use lib '.', $ENV{PERLGP_LIB} ||  die "
  PERLGP_LIB undefined
  please see README file concerning shell environment variables\n\n";

use Getopt::Long;
use Individual;
use Population;
use Cwd;
use POSIX qw(tmpnam);

my ($help);
GetOptions(help=>\$help);

my $frac = shift || 0.1;

die "usage: perlgp-sample-pop.pl [fraction of pop]\n" if ($help);

my ($exptid) = cwd() =~ m:([^/]+)$:;

my $population = new Population( ExperimentId => $exptid );

# and fill it from disk or tar file
$population->repopulate(RandomFraction=>$frac);

if ($population->countIndividuals()) {
  foreach $ind (@{$population->Individuals()}) {
    $ind->saveCode(Filename=>'-');
  }
} else {
  print "empty population\n";
}

